@@ -78,7 +78,7 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)res_init.c	8.1 (Berkeley) 6/7/93";
-static char rcsid[] = "$Id: res_init.c,v 1.5.88.1 2002/11/06 17:30:17 majka Exp $";
+static char rcsid[] = "$Id: res_init.c,v 1.8 2003/02/18 17:29:24 majka Exp $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/param.h>
@@ -86,11 +86,11 @@ static char rcsid[] = "$Id: res_init.c,v 1.5.88.1 2002/11/06 17:30:17 majka Exp
 #include <sys/time.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <arpa/nameser.h>
+#include <arpa/nameser8_compat.h>
 
 #include <stdio.h>
 #include <ctype.h>
-#include <resolv.h>
+#include <resolv8_compat.h>
 #if defined(BSD) && (BSD >= 199103)
 # include <unistd.h>
 # include <stdlib.h>
@@ -152,7 +152,6 @@ static u_int32_t net_mask __P((struct in_addr));
 extern
 #endif /* NeXT */
 struct __res_state _res;
-extern struct __res_state _res_shadow;
 
 /*
  * Set up default settings.  If the configuration file exist, the values
@@ -447,7 +446,6 @@ res_init()
 	if ((cp = getenv("RES_OPTIONS")) != NULL)
 		res_setoptions(cp, "env");
 	_res.options |= RES_INIT;
-	_res_shadow = _res;
 	return (0);
 }
 
